@@ -2,12 +2,16 @@ import copy
 import datetime
 from abc import ABC, abstractmethod
 from collections import defaultdict
+from typing import Generic, Union
 
 import pytz
+from typing_extensions import Self
 
-from fhirpy.base.resource import BaseResource, BaseReference
-from fhirpy.base.utils import chunks, encode_params
+from fhirpy.base.client import TClient
 from fhirpy.base.exceptions import InvalidResponse
+from fhirpy.base.resource import BaseReference, BaseResource
+from fhirpy.base.resource_protocol import TResource, get_resource_type_from_class
+from fhirpy.base.utils import chunks, encode_params
 
 FHIR_DATE_TIME_FORMAT = "%Y-%m-%dT%H:%M:%SZ"
 FHIR_DATE_FORMAT = "%Y-%m-%d"
@@ -56,13 +60,13 @@ def transform_value(value):
 
 
 class Raw:
-    kwargs = {}
+    kwargs: dict
 
     def __init__(self, **kwargs):
         self.kwargs = kwargs
 
 
-def SQ(*args, **kwargs):
+def SQ(*args, **kwargs):  # noqa: N802
     """
     Builds search query
 
@@ -131,8 +135,8 @@ def SQ(*args, **kwargs):
 
     res = defaultdict(list)
     for key, value in kwargs.items():
-        value = value if isinstance(value, list) else [value]
-        value = [transform_value(sub_value) for sub_value in value]
+        value = value if isinstance(value, list) else [value]  # noqa: PLW2901
+        value = [transform_value(sub_value) for sub_value in value]  # noqa: PLW2901
 
         key_parts = key.split("__")
 
@@ -153,15 +157,15 @@ def SQ(*args, **kwargs):
 
         if op:
             if op in param_ops:
-                param = "{0}:{1}".format(param, transform_param(op))
+                param = f"{param}:{transform_param(op)}"
             elif op in value_ops:
-                value = ["{0}{1}".format(op, sub_value) for sub_value in value]
+                value = [f"{op}{sub_value}" for sub_value in value]  # noqa: PLW2901
         res[transform_param(param)].extend(value)
 
     for arg in args:
         if isinstance(arg, Raw):
             for key, value in arg.kwargs.items():
-                value = value if isinstance(value, list) else [value]
+                value = value if isinstance(value, list) else [value]  # noqa: PLW2901
                 res[key].extend(value)
         else:
             raise ValueError("Can't handle args without Raw() wrapper")
@@ -169,70 +173,107 @@ def SQ(*args, **kwargs):
     return res
 
 
-class AbstractSearchSet(ABC):
-    client = None
-    resource_type = None
-    params = None
+class AbstractSearchSet(Generic[TClient, TResource], ABC):
+    client: TClient
+    resource_type: str
+    custom_resource_class: Union[type[TResource], None]
+    params: dict
 
-    def __init__(self, client, resource_type, params=None):
+    def __init__(
+        self,
+        client: TClient,
+        resource_type: Union[type[TResource], str],
+        params: Union[dict, None] = None,
+    ):
         self.client = client
-        self.resource_type = resource_type
+        self.resource_type = (
+            resource_type
+            if isinstance(resource_type, str)
+            else get_resource_type_from_class(resource_type)
+        )
+        self.custom_resource_class = None if isinstance(resource_type, str) else resource_type
         self.params = defaultdict(list, params or {})
 
-    def _perform_resource(self, data):
-        resource_type = data.get("resourceType", None)
-        resource = self.client.resource(resource_type, **data)
-        return resource
+    def _dict_to_resource(self, data) -> TResource:
+        if self.custom_resource_class and self.resource_type == data["resourceType"]:
+            return self.custom_resource_class(**data)
+        return self.client.resource(data["resourceType"], **data)
 
-    @abstractmethod  # pragma: no cover
+    @abstractmethod
+    def execute(
+        self,
+        path: str,
+        method: str = "post",
+        data: Union[dict, None] = None,
+        params: Union[dict, None] = None,
+    ):
+        pass
+
+    @abstractmethod
     def fetch(self):
         pass
 
-    @abstractmethod  # pragma: no cover
+    @abstractmethod
     def fetch_raw(self):
         pass
 
-    @abstractmethod  # pragma: no cover
+    @abstractmethod
     def fetch_all(self):
         pass
 
-    @abstractmethod  # pragma: no cover
-    def get(self, id):
+    @abstractmethod
+    def get(self, id):  # noqa: A002
         pass
 
-    @abstractmethod  # pragma: no cover
+    @abstractmethod
     def count(self):
         pass
 
-    @abstractmethod  # pragma: no cover
+    @abstractmethod
     def first(self):
         pass
 
-    def clone(self, override=False, **kwargs):
+    @abstractmethod
+    async def get_or_create(self, resource):
+        pass
+
+    @abstractmethod
+    def update(self, resource):
+        pass
+
+    @abstractmethod
+    def patch(self, _resource, **kwargs):
+        pass
+
+    def clone(self, override=False, **kwargs) -> Self:
         new_params = copy.deepcopy(self.params)
         for key, value in kwargs.items():
             if not isinstance(value, list):
-                value = [value]
+                value = [value]  # noqa: PLW2901
 
             if override:
                 new_params[key] = value
             else:
                 new_params[key].extend(value)
 
-        return self.__class__(self.client, self.resource_type, new_params)
+        return self.__class__(
+            self.client,
+            self.custom_resource_class if self.custom_resource_class else self.resource_type,
+            new_params,
+        )
 
-    def elements(self, *attrs, exclude=False):
-        attrs = set(attrs)
+    def elements(self, *attrs, exclude=False) -> Self:
+        attrs_set = set(attrs)
         if not exclude:
-            attrs |= {"id", "resourceType"}
-        attrs = [attr for attr in attrs]
+            attrs_set |= {"id", "resourceType"}
+        attrs_list = list(attrs_set)
 
         return self.clone(
-            _elements="{0}{1}".format("-" if exclude else "", ",".join(attrs)),
+            _elements="{}{}".format("-" if exclude else "", ",".join(attrs_list)),
             override=True,
         )
 
-    def has(self, *args, **kwargs):
+    def has(self, *args, **kwargs) -> Self:
         if len(args) % 2 != 0:
             raise TypeError(
                 "You should pass even size of arguments, for example: "
@@ -240,24 +281,22 @@ class AbstractSearchSet(ABC):
                 "'AuditEvent', 'entity', user='id')`"
             )
 
-        key_part = ":".join(
-            ["_has:{0}".format(":".join(pair)) for pair in chunks(args, 2)]
-        )
+        key_part = ":".join(["_has:{}".format(":".join(pair)) for pair in chunks(args, 2)])
 
         return self.clone(
             **{":".join([key_part, key]): value for key, value in SQ(**kwargs).items()}
         )
 
-    def include(
+    def include(  # noqa: PLR0913
         self,
-        resource_type,
-        attr=None,
-        target_resource_type=None,
+        resource_type: str,
+        attr: Union[str, None] = None,
+        target_resource_type: Union[str, None] = None,
         *,
         recursive=False,
         iterate=False,
-        reverse=False
-    ):
+        reverse=False,
+    ) -> Self:
         key_params = ["_revinclude" if reverse else "_include"]
 
         if iterate:
@@ -272,9 +311,7 @@ class AbstractSearchSet(ABC):
             value = "*"
         else:
             if not attr:
-                raise TypeError(
-                    "You should provide attr " "(search parameter) argument"
-                )
+                raise TypeError("You should provide attr (search parameter) argument")
             value_params = [resource_type, attr]
             if target_resource_type:
                 value_params.append(target_resource_type)
@@ -284,13 +321,13 @@ class AbstractSearchSet(ABC):
 
     def revinclude(
         self,
-        resource_type,
-        attr=None,
-        target_resource_type=None,
+        resource_type: str,
+        attr: Union[str, None] = None,
+        target_resource_type: Union[str, None] = None,
         *,
         recursive=False,
-        iterate=False
-    ):
+        iterate=False,
+    ) -> Self:
         return self.include(
             resource_type,
             attr=attr,
@@ -300,38 +337,33 @@ class AbstractSearchSet(ABC):
             reverse=True,
         )
 
-    def search(self, *args, **kwargs):
+    def search(self, *args, **kwargs) -> Self:
         return self.clone(**SQ(*args, **kwargs))
 
-    def limit(self, limit):
+    def limit(self, limit) -> Self:
         return self.clone(_count=limit, override=True)
 
-    def sort(self, *keys):
+    def sort(self, *keys) -> Self:
         sort_keys = ",".join(keys)
         return self.clone(_sort=sort_keys, override=True)
 
-    def __str__(self):  # pragma: no cover
-        return "<{0} {1}?{2}>".format(
-            self.__class__.__name__, self.resource_type, encode_params(self.params)
-        )
+    def __str__(self) -> str:
+        return f"<{self.__class__.__name__} {self.resource_type}?{encode_params(self.params)}>"
 
-    def __repr__(self):  # pragma: no cover
+    def __repr__(self) -> str:
         return self.__str__()
 
-    def _get_bundle_resources(self, bundle_data):
+    def _get_bundle_resources(self, bundle_data) -> list[TResource]:
         bundle_resource_type = bundle_data.get("resourceType", None)
 
         if bundle_resource_type != "Bundle":
-            raise InvalidResponse(
-                "Expected to receive Bundle "
-                "but {0} received".format(bundle_resource_type)
-            )
+            raise InvalidResponse(f"Expected to receive Bundle but {bundle_resource_type} received")
 
         resources_data = [res["resource"] for res in bundle_data.get("entry", [])]
 
         resources = []
         for data in resources_data:
-            resource = self._perform_resource(data)
-            if resource.resource_type == self.resource_type:
+            resource = self._dict_to_resource(data)
+            if resource.resourceType == self.resource_type:
                 resources.append(resource)
         return resources
